@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1992-2014 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1992-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -122,6 +122,7 @@ decayscreen_init (Display *dpy, Window window)
     st->random_p = 1;
     st->mode = random() % (FUZZ+1);
   }
+  if (s) free (s);
 
   st->delay = get_integer_resource (st->dpy, "delay", "Integer");
   if (st->delay < 0) st->delay = 0;
@@ -132,13 +133,10 @@ decayscreen_init (Display *dpy, Window window)
   XGetWindowAttributes (st->dpy, st->window, &st->xgwa);
 
   gcv.function = GXcopy;
-  gcv.subwindow_mode = IncludeInferiors;
   bg = get_pixel_resource (st->dpy, st->xgwa.colormap, "background", "Background");
   gcv.foreground = bg;
 
   gcflags = GCForeground | GCFunction;
-  if (use_subwindow_mode_p(st->xgwa.screen, st->window)) /* see grabscreen.c */
-    gcflags |= GCSubwindowMode;
   st->gc = XCreateGC (st->dpy, st->window, gcflags, &gcv);
 
   st->start_time = time ((time_t *) 0);
@@ -172,6 +170,9 @@ decayscreen_draw (Display *dpy, Window window, void *closure)
     static const int upright_bias[]   = { L,L,L,R, R,R,R,R, U,U,U,U, U,D,D,D };
     static const int downright_bias[] = { L,L,L,R, R,R,R,R, U,U,U,D, D,D,D,D };
 
+    int off = 1;
+    if (st->sizex > 2560 || st->sizey > 2560) off *= 2;  /* Retina displays */
+
     if (st->img_loader)   /* still loading */
       {
         st->img_loader = load_image_async_simple (st->img_loader, 
@@ -230,7 +231,7 @@ decayscreen_draw (Display *dpy, Window window, void *closure)
       width = nrnd( st->sizex/2 ) + st->sizex/2 - left;
       height = nrnd(st->sizey - top);
       toleft = left;
-      totop = top+1;
+      totop = top+off;
 
     } else if (st->mode == FUZZ) {  /* By Vince Levey <vincel@vincel.org>;
                                    inspired by the "melt" mode of the
@@ -242,7 +243,7 @@ decayscreen_draw (Display *dpy, Window window, void *closure)
       if (st->fuzz_toggle)
         {
           totop = top;
-          height = 1;
+          height = off;
           toleft = nrnd(st->sizex - 1);
           if (toleft > left)
             {
@@ -260,7 +261,7 @@ decayscreen_draw (Display *dpy, Window window, void *closure)
       else
         {
           toleft = left;
-          width = 1;
+          width = off;
           totop  = nrnd(st->sizey - 1);
           if (totop > top)
             {
@@ -304,19 +305,21 @@ decayscreen_draw (Display *dpy, Window window, void *closure)
       }
       
       switch (st->current_bias[random() % (sizeof(no_bias)/sizeof(*no_bias))]) {
-      case L: toleft = left-1; break;
-      case R: toleft = left+1; break;
-      case U: totop = top-1; break;
-      case D: totop = top+1; break;
+      case L: toleft = left-off; break;
+      case R: toleft = left+off; break;
+      case U: totop = top-off; break;
+      case D: totop = top+off; break;
       default: abort(); break;
       }
     }
     
     if (st->mode == STRETCH) {
-      XCopyArea (st->dpy, st->window, st->window, st->gc, 0, st->sizey-top-2, st->sizex, top+1, 
-		 0, st->sizey-top-1); 
+      XCopyArea (st->dpy, st->window, st->window, st->gc,
+                 0, st->sizey-top-off*2, st->sizex, top+off, 
+		 0, st->sizey-top-off);
     } else {
-      XCopyArea (st->dpy, st->window, st->window, st->gc, left, top, width, height,
+      XCopyArea (st->dpy, st->window, st->window, st->gc,
+                 left, top, width, height,
 		 toleft, totop);
     }
 
@@ -356,6 +359,7 @@ static void
 decayscreen_free (Display *dpy, Window window, void *closure)
 {
   struct state *st = (struct state *) closure;
+  XFreeGC (dpy, st->gc);
   free (st);
 }
 
@@ -366,11 +370,6 @@ static const char *decayscreen_defaults [] = {
   ".foreground:			Yellow",
   "*dontClearRoot:		True",
   "*fpsSolid:			True",
-
-#ifdef __sgi	/* really, HAVE_READ_DISPLAY_EXTENSION */
-  "*visualID:			Best",
-#endif
-
   "*delay:			10000",
   "*mode:			random",
   "*duration:			120",
