@@ -14,10 +14,7 @@
 			"*showFPS:      False       \n" \
 			"*wireframe:    False       \n" \
 
-# define refresh_ball 0
 # define release_ball 0
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
 
 #include "xlockmore.h"
 #include "normals.h"
@@ -82,7 +79,7 @@ ENTRYPOINT ModeSpecOpt ball_opts = {countof(opts), opts, countof(vars), vars, NU
 static XYZ
 normalize (XYZ p)
 {
-  GLfloat d = sqrt(p.x*p.x + p.y*p.y * p.z*p.z);
+  GLfloat d = sqrt(p.x*p.x + p.y*p.y + p.z*p.z);
   if (d < 0.0000001)
     p.x = p.y = p.z = 0;
   else
@@ -499,8 +496,15 @@ ENTRYPOINT void
 reshape_ball (ModeInfo *mi, int width, int height)
 {
   GLfloat h = (GLfloat) height / (GLfloat) width;
+  int y = 0;
 
-  glViewport (0, 0, (GLint) width, (GLint) height);
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport (0, y, (GLint) width, (GLint) height);
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -512,13 +516,12 @@ reshape_ball (ModeInfo *mi, int width, int height)
              0.0, 0.0, 0.0,
              0.0, 1.0, 0.0);
 
-# ifdef HAVE_MOBILE	/* Keep it the same relative size when rotated. */
   {
-    int o = (int) current_device_rotation();
-    if (o != 0 && o != 180 && o != -180)
-      glScalef (1/h, 1/h, 1/h);
+    GLfloat s = (MI_WIDTH(mi) < MI_HEIGHT(mi)
+                 ? (MI_WIDTH(mi) / (GLfloat) MI_HEIGHT(mi))
+                 : 1);
+    glScalef (s, s, s);
   }
-# endif
 
   glClear(GL_COLOR_BUFFER_BIT);
 }
@@ -538,16 +541,13 @@ ball_handle_event (ModeInfo *mi, XEvent *event)
 }
 
 
-static void free_ball (ModeInfo *mi);
-
-
 ENTRYPOINT void 
 init_ball (ModeInfo *mi)
 {
   ball_configuration *bp;
   int wire = MI_IS_WIREFRAME(mi);
 
-  MI_INIT (mi, bps, free_ball);
+  MI_INIT (mi, bps);
 
   bp = &bps[MI_SCREEN(mi)];
 
@@ -634,7 +634,7 @@ draw_ball (ModeInfo *mi)
   if (!bp->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(bp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
 
   glShadeModel(GL_SMOOTH);
 
@@ -686,17 +686,21 @@ draw_ball (ModeInfo *mi)
 }
 
 
-static void
+ENTRYPOINT void
 free_ball (ModeInfo *mi)
 {
   ball_configuration *bp = &bps[MI_SCREEN(mi)];
+  if (!bp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
   while (bp->tiles)
     {
       tile *t = bp->tiles->next;
       free (bp->tiles);
       bp->tiles = t;
     }
-  free (bp->rays);
+  if (bp->rays) free (bp->rays);
+  if (bp->trackball) gltrackball_free (bp->trackball);
+  if (bp->rot) free_rotator (bp->rot);
 }
 
 XSCREENSAVER_MODULE_2 ("Discoball", discoball, ball)
